package com.jh.boot.attachment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.nio.file.Paths;
import java.util.Objects;

import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Value;

public class FileBasedAttachmentService extends AbstractHibernateAttachmentService {

	private String dataDirectory;
	
	@Override
	public InputStream fetchAttachmentBody(Attachment attachment) {
		try {
			return new FileInputStream(file(attachment));
		} catch (FileNotFoundException e) {
			throw new RuntimeException("File not found", e);
		}
	}

	@Override
	protected void saveData(Attachment attachment, InputStream stream) {
		try (FileOutputStream fos = new FileOutputStream(file(attachment))) {
			IOUtils.copy(stream, fos);
		} catch (Exception e) {
			throw new RuntimeException("Cannot write data", e);
		}
	}
	

	@Override
	protected void deleteData(Attachment attachment) {
		File file = file(attachment);
		if (file.exists()) {
			file.delete();
			
			file = file.getParentFile();
			if (file.listFiles().length == 0) {
				//delete ?
				file.delete();
			}
		}
	}
	
	protected File file(Attachment attachment) {
		Objects.requireNonNull(attachment, "Attachment must not be null");
		File file = Paths.get(dataDirectory, s(attachment.getUnitId()), attachment.getObjectType(), s(attachment.getObjectId()), attachment.getFilename()).toFile();
		file.getParentFile().mkdirs();
		return file;
	}

	
	protected String s(Number n) {
		return n == null ? "0" : String.valueOf(n);
	}
	
	@Value("${attachment.store.path:/var/data}")
	public void setDataDirectory(String dataDirectory) {
		this.dataDirectory = dataDirectory;
	}


}
